from pygame import joystick, event, display, constants
from tkinter import (
    Frame,
    Label,
    Button,
    StringVar,
    BooleanVar,
    IntVar,
    Radiobutton,
    Canvas,
    OptionMenu,
    messagebox,
)
from toml import load, dumps
from time import sleep
from os import path, stat
from shutil import copyfile

from src.joypyviewer.arg import get_args
from src.joypyviewer.gui import create_gui

joypyargs = get_args()
tkgui = create_gui()

default_fill_color = "deeppink"
default_outline_color = "deeppink"
default_chroma_color = "#00ff00"

if joypyargs.background:
    default_chroma_color = joypyargs.background

if joypyargs.color:
    default_fill_color = joypyargs.color
    default_outline_color = joypyargs.color

module_path = path.dirname(path.abspath(__file__))

display.init()

show_events = BooleanVar()
show_events.set(True)

allowed_event_types = [
    constants.JOYBUTTONUP,
    constants.JOYBUTTONDOWN,
    constants.JOYHATMOTION,
    constants.JOYAXISMOTION,
    constants.JOYDEVICEREMOVED,
    constants.JOYDEVICEADDED,
]

current_joys = dict()

quantity_joys_label = StringVar()

joy0 = StringVar()
joy1 = StringVar()
joy2 = StringVar()
joy3 = StringVar()

joy_guid = StringVar()
joy_namelabel = StringVar()
joy_guidlabel = StringVar()

numhats = IntVar()

pressed_button = IntVar()

selected_joystick = IntVar()

emu_name = StringVar()

dummy_mappings = {
    "button_1": "false",
    "button_2": "false",
    "button_a": "false",
    "button_b": "false",
    "button_c": "false",
    "button_l": "false",
    "button_r": "false",
    "button_x": "false",
    "button_y": "false",
    "button_z": "false",
    "button_select": "false",
    "button_start": "false",
}

emulators = {
    "NES": {
        "button_a": "false",
        "button_b": "false",
        "button_select": "false",
        "button_start": "false",
    },
    "Super Nintendo": {
        "button_a": "false",
        "button_b": "false",
        "button_x": "false",
        "button_y": "false",
        "button_l": "false",
        "button_r": "false",
        "button_select": "false",
        "button_start": "false",
    },
    "Master System": {"button_1": "false", "button_2": "false"},
    "Genesis 3-button": {
        "button_a": "false",
        "button_b": "false",
        "button_c": "false",
        "button_start": "false",
    },
    "Genesis 6-button": {
        "button_a": "false",
        "button_b": "false",
        "button_c": "false",
        "button_x": "false",
        "button_y": "false",
        "button_z": "false",
        "button_start": "false",
    },
}

emu_name.set("NES")


def copy_to_clipboard():
    tkgui.clipboard_clear()
    tkgui.clipboard_append(joy_guid.get())


def stop():
    pressed_button.set("")
    current_joys.clear()
    joystick.quit()


def clean_quit():
    show_events.set(False)
    stop()
    tkgui.destroy()


def ask_quit():
    result = messagebox.askyesno("Quitting...", "Wanna quit?")
    if result == 1:
        clean_quit()


def create_and_load_config(guid, emu):

    sample_conf_path = path.join(module_path, "config/joypyviewer.sample.conf")
    home_dir = path.expanduser("~")
    joypyviewer_conf_name = ".joypyviewer.conf"
    joypyviewer_conf = home_dir + "/" + joypyviewer_conf_name

    if not path.exists(joypyviewer_conf):
        copyfile(sample_conf_path, joypyviewer_conf)
    elif stat(joypyviewer_conf).st_size == 0:
        copyfile(sample_conf_path, joypyviewer_conf)

    if joypyargs.config:
        text = load(joypyargs.config.name)
        return text
    else:
        with open(joypyviewer_conf, "a+") as filecfg:

            wrk_joy = current_joys.get(selected_joystick.get())

            filecfg.seek(0)
            readfile = filecfg.read()

            if guid not in readfile:

                new_dict = dict()
                final = dict()

                for emu in emulators.keys():
                    new_dict[emu] = emulators.get(emu)

                final[guid] = new_dict

                toml_text = dumps(final)

                filecfg.write(str("\n#") + wrk_joy + str("\n"))
                filecfg.write(toml_text)

            #############################

            line_counter = 0
            specific_line = ""

            filecfg.seek(0)
            readlines = filecfg.readlines()

            lines = ""
            sum_dict = dict()
            text_to_write = dict()

            for line in readlines:

                line_counter += 1

                if wrk_joy in line:
                    specific_line = line_counter

                if guid in line:
                    lines += line.strip("\n")

            if emu not in lines:
                sum_dict[emu] = emulators.get(emu)

                text_to_write[guid] = sum_dict

                text = dumps(text_to_write)
                filecfg.write(str("\n#") + wrk_joy)
                filecfg.write("\n" + text)

        loaded_cfg = load(joypyviewer_conf)
        return loaded_cfg


def init(arg=None):
    def reset_sms_hat():
        if emu == "Master System":
            can.itemconfig(draw_top_right, fill=outlinecolornorm)
            can.itemconfig(draw_top_left, fill=outlinecolornorm)
            can.itemconfig(draw_bottom_left, fill=outlinecolornorm)
            can.itemconfig(draw_bottom_right, fill=outlinecolornorm)

    def press_dpad():

        if event_value == (0, 0):
            can.itemconfig(draw_up, fill=fillcolornorm, outline=outlinecolornorm)
            can.itemconfig(draw_down, fill=fillcolornorm, outline=outlinecolornorm)
            can.itemconfig(draw_right, fill=fillcolornorm, outline=outlinecolornorm)
            can.itemconfig(draw_left, fill=fillcolornorm, outline=outlinecolornorm)
            reset_sms_hat()

        if event_value == (0, 1):
            can.itemconfig(draw_up, fill=fillcolor, outline=outlinecolor)
            can.itemconfig(draw_left, fill=fillcolornorm, outline=outlinecolornorm)
            can.itemconfig(draw_right, fill=fillcolornorm, outline=outlinecolornorm)
            can.itemconfig(draw_down, fill=fillcolornorm, outline=outlinecolornorm)
            reset_sms_hat()

        if event_value == (1, 0):
            can.itemconfig(draw_right, fill=fillcolor, outline=outlinecolor)
            can.itemconfig(draw_left, fill=fillcolornorm, outline=outlinecolornorm)
            can.itemconfig(draw_up, fill=fillcolornorm, outline=outlinecolornorm)
            can.itemconfig(draw_down, fill=fillcolornorm, outline=outlinecolornorm)
            reset_sms_hat()

        if event_value == (0, -1):
            can.itemconfig(draw_down, fill=fillcolor, outline=outlinecolor)
            can.itemconfig(draw_left, fill=fillcolornorm, outline=outlinecolornorm)
            can.itemconfig(draw_right, fill=fillcolornorm, outline=outlinecolornorm)
            can.itemconfig(draw_up, fill=fillcolornorm, outline=outlinecolornorm)
            reset_sms_hat()

        if event_value == (-1, 0):
            can.itemconfig(draw_left, fill=fillcolor, outline=outlinecolor)
            can.itemconfig(draw_up, fill=fillcolornorm, outline=outlinecolornorm)
            can.itemconfig(draw_right, fill=fillcolornorm, outline=outlinecolornorm)
            can.itemconfig(draw_down, fill=fillcolornorm, outline=outlinecolornorm)
            reset_sms_hat()

        if event_value == (1, 1):
            can.itemconfig(draw_up, fill=fillcolor, outline=outlinecolor)
            can.itemconfig(draw_right, fill=fillcolor, outline=outlinecolor)

            if emu == "Master System":
                can.itemconfig(draw_top_right, fill=outlinecolor)

        if event_value == (-1, 1):
            can.itemconfig(draw_left, fill=fillcolor, outline=outlinecolor)
            can.itemconfig(draw_up, fill=fillcolor, outline=outlinecolor)

            if emu == "Master System":
                can.itemconfig(draw_top_left, fill=outlinecolor)

        if event_value == (1, -1):
            can.itemconfig(draw_down, fill=fillcolor, outline=outlinecolor)
            can.itemconfig(draw_right, fill=fillcolor, outline=outlinecolor)

            if emu == "Master System":
                can.itemconfig(draw_bottom_right, fill=outlinecolor)

        if event_value == (-1, -1):
            can.itemconfig(draw_left, fill=fillcolor, outline=outlinecolor)
            can.itemconfig(draw_down, fill=fillcolor, outline=outlinecolor)

            if emu == "Master System":
                can.itemconfig(draw_bottom_left, fill=outlinecolor)

    def press_axis():

        if event_axis == 0:
            if event_value == 1.0:
                can.itemconfig(draw_right, fill=fillcolor, outline=outlinecolor)

            if event_value == -1.000030518509476:
                can.itemconfig(draw_left, fill=fillcolor, outline=outlinecolor)

            if event_value == -0.003936887722403638:
                can.itemconfig(draw_left, fill=fillcolornorm, outline=outlinecolornorm)
                can.itemconfig(draw_right, fill=fillcolornorm, outline=outlinecolornorm)

        if event_axis == 1:
            if event_value == 1.0:
                can.itemconfig(draw_down, fill=fillcolor, outline=outlinecolor)

            if event_value == -1.000030518509476:
                can.itemconfig(draw_up, fill=fillcolor, outline=outlinecolor)

            if event_value == -0.003936887722403638:
                can.itemconfig(draw_up, fill=fillcolornorm, outline=outlinecolornorm)
                can.itemconfig(draw_down, fill=fillcolornorm, outline=outlinecolornorm)

    def press_buttons():

        if etype == constants.JOYBUTTONDOWN:
            pressed_button.set(event_button)

        if etype == constants.JOYBUTTONUP:
            pressed_button.set("")

        if event_button == button_a:
            can.itemconfig(draw_a, fill=fillcolor, outline=outlinecolor)

        if event_button == button_b:
            can.itemconfig(draw_b, fill=fillcolor, outline=outlinecolor)

        if event_button == button_c:
            can.itemconfig(draw_c, fill=fillcolor, outline=outlinecolor)

        if event_button == button_start:
            for i in draw_start:
                can.itemconfig(i, fill=fillcolor, outline=outlinecolor)

        if event_button == button_x:
            can.itemconfig(draw_x, fill=fillcolor, outline=outlinecolor)

        if event_button == button_y:
            can.itemconfig(draw_y, fill=fillcolor, outline=outlinecolor)

        if event_button == button_1:
            can.itemconfig(draw_1, fill=fillcolor, outline=outlinecolor)

        if event_button == button_2:
            can.itemconfig(draw_2, fill=fillcolor, outline=outlinecolor)

        if event_button == button_z:
            can.itemconfig(draw_z, fill=fillcolor, outline=outlinecolor)

        if event_button == button_select:
            for i in draw_select:
                can.itemconfig(i, fill=fillcolor, outline=outlinecolor)

        if event_button == button_l:
            if type(draw_l) == list:
                for i in draw_l:
                    can.itemconfig(i, fill=fillcolor, outline=outlinecolor)
            else:
                can.itemconfig(draw_l, fill=fillcolor, outline=outlinecolor)

        if event_button == button_r:
            if type(draw_r) == list:
                for i in draw_r:
                    can.itemconfig(i, fill=fillcolor, outline=outlinecolor)
            else:
                can.itemconfig(draw_r, fill=fillcolor, outline=outlinecolor)

    stop()

    joystick.init()

    num = selected_joystick.get()
    emu = emu_name.get()

    joy_count = joystick.get_count()
    quantity_joys_label.set("Connected joysticks: " + str(joy_count))

    for i in range(4):
        globals()["joy%s" % str(i)].set("not found")
        globals()["Butt%s" % str(i + 1)].config(state="disabled")

    for i in range(joy_count):
        globals()["Butt%s" % str(i + 1)].config(state="normal")

    if joy_count > 0:

        if not BottomInsideFrameRight.winfo_children():

            JoynameLabel = Label(BottomInsideFrameRight, textvariable=joy_namelabel)
            JoynameLabel.pack(anchor="e")

            GuidLabel = Label(BottomInsideFrameRight, textvariable=joy_guidlabel)
            GuidLabel.pack(anchor="e")

            CopyButton = Button(BottomInsideFrameRight)
            CopyButton.config(text="Copy GUID to Clipboard", command=copy_to_clipboard)
            CopyButton.pack(anchor="e")

        if not BottomInsideFrameLeft.winfo_children():

            PressedButton1 = Label(BottomInsideFrameLeft, text="Pressed button:")
            PressedButton1.pack(side="left", anchor="sw")

            PressedButton2 = Label(BottomInsideFrameLeft, textvariable=pressed_button)
            PressedButton2.pack(side="left", anchor="sw")

        for x in range(joy_count):
            joy = joystick.Joystick(x).get_name()

            current_joys[x] = joy
            globals()["joy%s" % str(x)].set(current_joys[x])

        oldlabel = globals()["joy%s" % str(num)].get()

        if len(joy0.get()) > 25:
            joy0.set(joy0.get()[:25] + "...")

        if len(joy1.get()) > 25:
            joy1.set(joy1.get()[:25] + "...")

        if len(joy2.get()) > 25:
            joy2.set(joy2.get()[:25] + "...")

        if len(joy3.get()) > 25:
            joy3.set(joy3.get()[:25] + "...")

        if current_joys.get(num) != oldlabel:
            selected_joystick.set(0)
            joy_obj = joystick.Joystick(0)
        else:
            joy_obj = joystick.Joystick(num)

        joy_obj.init()

        numhats.set(joy_obj.get_numhats())
        joy_guid.set(joy_obj.get_guid())
        guid = joy_guid.get()

        joy_namelabel.set(joy_obj.get_name())
        joy_guidlabel.set("GUID: " + guid)

        alljoysconfig = create_and_load_config(guid, emu)
        singlejoymappings = alljoysconfig.get(guid)

        emu_buttons = dict()
        emu_buttons.update(dummy_mappings)

        if emu not in singlejoymappings.keys():
            emu_buttons.update(emulators.get(emu))
        else:
            emu_buttons.update(singlejoymappings.get(emu))

        for key, value in emu_buttons.items():
            if value == "false":
                emu_buttons[key] = "-0"

        globals().update(emu_buttons)

        fillcolornorm = "#454545"
        outlinecolornorm = "#454545"

        can.delete("all")

        if emu == "NES":
            layout_bg1 = can.create_rectangle(
                4, 57, 374, 217, fill="#cececb", outline="#cececb"
            )
            layout_bg2 = can.create_rectangle(
                16, 83, 362, 205, fill="#404040", outline="#404040"
            )
            layout_hat_bg_hor = can.create_rectangle(
                32, 135, 108, 167, fill="#d4d4d2", outline="#d4d4d2"
            )
            layout_hat_bg_ver = can.create_rectangle(
                55, 113, 85, 189, fill="#d4d4d2", outline="#d4d4d2"
            )
            layout_hat_bg_middle = can.create_rectangle(
                58, 139, 82, 163, fill="#454545", outline="#454545"
            )
            layout_start_select_bg = can.create_rectangle(
                127, 155, 223, 186, fill="#d4d4d2", outline="#d4d4d2"
            )
            layout_stick1_bg = can.create_rectangle(
                127, 129, 223, 146, fill="#9e9e9a", outline="#9e9e9a"
            )
            layout_stick2_bg = can.create_rectangle(
                127, 106, 223, 123, fill="#9e9e9a", outline="#9e9e9a"
            )
            layout_stick3_bg = can.create_rectangle(
                127, 83, 223, 100, fill="#9e9e9a", outline="#9e9e9a"
            )
            layout_stick4_bg = can.create_rectangle(
                127, 195, 223, 205, fill="#9e9e9a", outline="#9e9e9a"
            )
            layout_bg_button_a = can.create_rectangle(
                243, 149, 285, 190, fill="#d4d4d2", outline="#d4d4d2"
            )
            layout_bg_button_b = can.create_rectangle(
                292, 149, 332, 190, fill="#d4d4d2", outline="#d4d4d2"
            )
            draw_up = can.create_rectangle(
                58, 117, 82, 138, fill=fillcolornorm, outline=outlinecolornorm
            )
            draw_left = can.create_rectangle(
                36, 139, 57, 163, fill=fillcolornorm, outline=outlinecolornorm
            )
            draw_right = can.create_rectangle(
                83, 139, 104, 163, fill=fillcolornorm, outline=outlinecolornorm
            )
            draw_down = can.create_rectangle(
                58, 164, 82, 185, fill=fillcolornorm, outline=outlinecolornorm
            )
            draw_a = can.create_oval(
                295, 153, 328, 186, fill=fillcolornorm, outline=outlinecolornorm
            )
            draw_b = can.create_oval(
                247, 153, 280, 186, fill=fillcolornorm, outline=outlinecolornorm
            )
            draw_select = [
                can.create_oval(
                    136, 164, 148, 175, fill=fillcolornorm, outline=outlinecolornorm
                ),
                can.create_rectangle(
                    140, 164, 158, 175, fill=fillcolornorm, outline=outlinecolornorm
                ),
                can.create_oval(
                    154, 164, 166, 175, fill=fillcolornorm, outline=outlinecolornorm
                ),
            ]
            draw_start = [
                can.create_oval(
                    184, 164, 196, 175, fill=fillcolornorm, outline=outlinecolornorm
                ),
                can.create_rectangle(
                    190, 164, 210, 175, fill=fillcolornorm, outline=outlinecolornorm
                ),
                can.create_oval(
                    202, 164, 214, 175, fill=fillcolornorm, outline=outlinecolornorm
                ),
            ]
            can.create_text(149, 138, text="SELECT", font="Sans 7", fill="#b7433d")
            can.create_text(203, 138, text="START", font="Sans 7", fill="#b7433d")
            can.create_text(328, 199, text="A", font="Sans 11", fill="#b7433d")
            can.create_text(279, 199, text="B", font="Sans 11", fill="#b7433d")
        elif emu == "Super Nintendo":
            draw_l = [
                can.create_rectangle(
                    69, 60, 118, 66, fill=fillcolornorm, outline=outlinecolornorm
                ),
                can.create_oval(
                    111, 60, 128, 66, fill=fillcolornorm, outline=outlinecolornorm
                ),
                can.create_oval(
                    44, 60, 107, 87, fill=fillcolornorm, outline=outlinecolornorm
                ),
            ]
            draw_r = [
                can.create_rectangle(
                    265, 60, 305, 66, fill=fillcolornorm, outline=outlinecolornorm
                ),
                can.create_oval(
                    251, 60, 272, 66, fill=fillcolornorm, outline=outlinecolornorm
                ),
                can.create_oval(
                    264, 60, 336, 87, fill=fillcolornorm, outline=outlinecolornorm
                ),
            ]
            layout_bg_rectangle = can.create_rectangle(
                92, 66, 289, 205, fill="#cdcdd9", outline="#cdcdd9"
            )
            layout_circle_left = can.create_oval(
                6, 66, 162, 224, fill="#cdcdd9", outline="#cdcdd9"
            )
            layout_circle_right = can.create_oval(
                219, 66, 375, 224, fill="#cdcdd9", outline="#cdcdd9"
            )
            layout_circle_right2 = can.create_oval(
                230, 79, 366, 216, fill="#8e8ea2", outline="#8e8ea2"
            )
            layout_circle_hat = can.create_oval(42, 105, 128, 191, outline="#d6d6e0")
            layout_bg_hat = can.create_rectangle(
                76, 139, 94, 157, fill="#454545", outline="#454545"
            )
            draw_up = can.create_rectangle(
                76, 116, 94, 138, fill="#454545", outline="#454545"
            )
            draw_left = can.create_rectangle(
                53, 139, 75, 157, fill="#454545", outline="#454545"
            )
            draw_right = can.create_rectangle(
                95, 139, 117, 157, fill="#454545", outline="#454545"
            )
            draw_down = can.create_rectangle(
                76, 158, 94, 180, fill="#454545", outline="#454545"
            )
            x = 0
            y = 0
            for i in range(11):
                x1 = 244 + x
                y1 = 133 - y
                x2 = 279 + x
                y2 = 167 - y
                x += 4
                y += 3
                can.create_oval(x1, y1, x2, y2, fill="#c4c4d1", outline="#c4c4d1")
            x = 0
            y = 0
            for i in range(11):
                x1 = 280 + x
                y1 = 159 - y
                x2 = 314 + x
                y2 = 194 - y
                x += 4
                y += 3
                can.create_oval(x1, y1, x2, y2, fill="#c4c4d1", outline="#c4c4d1")
            x = 0
            y = 0
            draw_start = []
            for i in range(5):
                x1 = 183 + x
                y1 = 159 - y
                x2 = 197 + x
                y2 = 172 - y
                x += 4
                y += 3
                draw_start.append(
                    can.create_oval(
                        x1, y1, x2, y2, fill=fillcolornorm, outline=outlinecolornorm
                    )
                )
            x = 0
            y = 0
            draw_select = []
            for i in range(5):
                x1 = 143 + x
                y1 = 159 - y
                x2 = 157 + x
                y2 = 172 - y
                x += 4
                y += 3
                draw_select.append(
                    can.create_oval(
                        x1, y1, x2, y2, fill=fillcolornorm, outline=outlinecolornorm
                    )
                )
            draw_a = can.create_oval(
                319, 134, 348, 163, fill=fillcolornorm, outline=outlinecolornorm
            )
            draw_b = can.create_oval(
                284, 162, 312, 190, fill=fillcolornorm, outline=outlinecolornorm
            )
            draw_y = can.create_oval(
                248, 134, 278, 164, fill=fillcolornorm, outline=outlinecolornorm
            )
            draw_x = can.create_oval(
                284, 107, 313, 136, fill=fillcolornorm, outline=outlinecolornorm
            )
            can.create_text(323, 105, text="X", font="Sans 11", fill="#c8c8cc")
            can.create_text(359, 133, text="A", font="Sans 11", fill="#c8c8cc")
            can.create_text(241, 166, text="Y", font="Sans 11", fill="#c8c8cc")
            can.create_text(276, 193, text="B", font="Sans 11", fill="#c8c8cc")
            can.create_text(158, 184, text="SELECT", font="Sans 6", fill="grey")
            can.create_text(201, 184, text="START", font="Sans 6", fill="grey")
        elif emu == "Genesis 3-button":
            layout_bg_top = can.create_oval(
                22, 32, 353, 182, fill="#171717", outline="#171717"
            )
            layout_bg_right1 = can.create_oval(
                250, 58, 344, 248, fill="#171717", outline="#171717"
            )
            layout_bg_rectangle = can.create_rectangle(
                108, 171, 268, 220, fill="#171717", outline="#171717"
            )
            layout_bg_rectangle2 = can.create_oval(
                111,
                182,
                263,
                265,
                fill=default_chroma_color,
                outline=default_chroma_color,
            )
            layout_bg_right2 = can.create_oval(
                279, 73, 364, 213, fill="#171717", outline="#171717"
            )
            layout_bg_right3 = can.create_oval(
                261, 105, 351, 233, fill="#171717", outline="#171717"
            )
            layout_bg_left1 = can.create_oval(
                32, 64, 126, 242, fill="#171717", outline="#171717"
            )
            layout_bg_left2 = can.create_oval(
                14, 66, 108, 213, fill="#171717", outline="#171717"
            )
            layout_bg_left3 = can.create_oval(
                28, 118, 117, 229, fill="#171717", outline="#171717"
            )
            layout_bg_left4 = can.create_oval(
                36, 127, 117, 239, fill="#171717", outline="#171717"
            )
            layout_circle_hat1 = can.create_oval(36, 73, 148, 185, outline="#202020")
            layout_circle_hat2 = can.create_oval(49, 85, 135, 172, outline="#202020")
            layout_circle_hat3 = can.create_oval(
                59, 95, 124, 161, fill="#242424", outline="#121212", width=3
            )

            layout_center_hat = can.create_rectangle(
                85, 121, 98, 134, fill=fillcolornorm, outline=fillcolornorm
            )
            layout_center_hat = can.create_oval(
                86, 122, 97, 133, fill="#303030", outline="#303030"
            )
            circle_top_left = can.create_oval(
                128, 161, 131, 164, fill="#fefefe", outline="white"
            )
            circle_top_right = can.create_oval(
                126, 90, 129, 93, fill="#fefefe", outline="white"
            )

            circle_bottom_left = can.create_oval(
                55, 161, 58, 164, fill="#fefefe", outline="white"
            )
            circle_bottom_right = can.create_oval(
                55, 90, 58, 93, fill="#fefefe", outline="white"
            )

            arrow_right = can.create_line(
                146, 127, 147, 127, fill="#fefefe", arrow="last"
            )
            arrow_up = can.create_line(92, 75, 92, 74, fill="#fefefe", arrow="last")

            arrow_down = can.create_line(92, 183, 92, 184, fill="#fefefe", arrow="last")
            arrow_left = can.create_line(37, 128, 36, 128, fill="#fefefe", arrow="last")

            draw_left = can.create_rectangle(
                62, 121, 84, 135, fill=fillcolornorm, outline=outlinecolornorm
            )
            draw_right = can.create_rectangle(
                99, 121, 121, 135, fill=fillcolornorm, outline=outlinecolornorm
            )
            draw_up = can.create_rectangle(
                84, 98, 98, 120, fill=fillcolornorm, outline=outlinecolornorm
            )
            draw_down = can.create_rectangle(
                85, 135, 99, 157, fill=fillcolornorm, outline=outlinecolornorm
            )
            draw_a = can.create_oval(
                234, 135, 268, 169, fill=fillcolornorm, outline=fillcolornorm
            )
            draw_b = can.create_oval(
                272, 118, 306, 152, fill=fillcolornorm, outline=fillcolornorm
            )
            draw_c = can.create_oval(
                310, 104, 344, 138, fill=fillcolornorm, outline=fillcolornorm
            )
            x = 0
            y = 0
            draw_start = []
            for i in range(6):
                x1 = 250 + x
                y1 = 84 - y
                x2 = 259 + x
                y2 = 93 - y
                x += 5
                y += 2
                draw_start.append(
                    can.create_oval(
                        x1, y1, x2, y2, fill=fillcolornorm, outline=outlinecolornorm
                    )
                )
            can.create_text(251, 153, text="A", font="Sans 11", fill="#9b2d32")
            can.create_text(290, 135, text="B", font="Sans 11", fill="#9b2d32")
            can.create_text(328, 121, text="C", font="Sans 11", fill="#9b2d32")
            can.create_text(257, 77, text="S", font="Sans 5", fill="white")
            can.create_text(261, 76, text="T", font="Sans 5", fill="white")
            can.create_text(265, 74, text="A", font="Sans 5", fill="white")
            can.create_text(269, 72, text="R", font="Sans 5", fill="white")
            can.create_text(273, 71, text="T", font="Sans 5", fill="white")
            can.create_text(266, 114, text="T", font="Sans 5", fill="white")
            can.create_text(271, 112, text="R", font="Sans 5", fill="white")
            can.create_text(275, 110, text="I", font="Sans 5", fill="white")
            can.create_text(278, 109, text="G", font="Sans 5", fill="white")
            can.create_text(283, 107, text="G", font="Sans 5", fill="white")
            can.create_text(288, 105, text="E", font="Sans 5", fill="white")
            can.create_text(293, 103, text="R", font="Sans 5", fill="white")
        elif emu == "Master System":
            layout_background = can.create_rectangle(
                7, 60, 371, 208, fill="#504f53", outline="#504f53"
            )
            layout_dpad_bg = can.create_rectangle(
                32, 65, 165, 203, fill="#333333", outline="#333333"
            )
            layout_dpad_bg_right = can.create_rectangle(
                163, 108, 362, 182, fill="#333333", outline="#333333"
            )
            layout_dpad_bg_bottom = can.create_rectangle(
                31, 206, 171, 207, fill="#303030", outline="#303030"
            )
            layout_dpad_bg_bottom2 = can.create_rectangle(
                170, 186, 173, 207, fill="#303030", outline="#303030"
            )
            layout_dpad_bg_bottom3 = can.create_rectangle(
                174, 186, 371, 190, fill="#303030", outline="#303030"
            )
            layout_dpad_bg_top1 = can.create_rectangle(
                31, 60, 171, 62, fill="#303030", outline="#303030"
            )
            layout_dpad_bg_top2 = can.create_rectangle(
                170, 60, 173, 104, fill="#303030", outline="#303030"
            )
            layout_dpad_bg_top3 = can.create_rectangle(
                174, 98, 371, 104, fill="#303030", outline="#303030"
            )
            layout_grey_rectangle = can.create_rectangle(
                227, 125, 283, 177, outline="#65636C", fill="#65636C"
            )
            layout_grey_rectangle2 = can.create_rectangle(
                291, 125, 346, 177, outline="#65636C", fill="#65636C"
            )
            layout_red_rectangle = can.create_rectangle(
                227, 111, 283, 121, fill="#8A4746", outline="#8A4746"
            )
            layout_red_rectangle2 = can.create_rectangle(
                291, 111, 346, 121, fill="#8A4746", outline="#8A4746"
            )
            layout_button1_outside = can.create_oval(
                233, 129, 278, 173, fill="#242424", outline="#242424"
            )
            layout_button2_outside = can.create_oval(
                296, 129, 340, 173, fill="#242424", outline="#242424"
            )

            x_pos = 165
            for i in range(15):
                can.create_line(x_pos, 125, x_pos, 178, fill="#545454")
                x_pos += 4

            layout_red_box = can.create_rectangle(
                38, 71, 157, 195, width="3", outline="#8A4746"
            )
            layout_red_delete = can.create_line(
                53, 71, 140, 71, width="3", fill="#252525"
            )
            layout_greybg = can.create_rectangle(
                45, 78, 149, 188, fill="grey", outline="grey"
            )
            layout_black_left = can.create_rectangle(
                25, 60, 30, 208, fill="#252525", outline="#252525"
            )
            layout_dpad_delete1 = can.create_line(
                44, 77, 65, 99, fill="#333333", width="3"
            )
            layout_dpad_delete2 = can.create_line(
                114, 110, 150, 76, fill="#333333", width="3"
            )
            layout_dpad_delete3 = can.create_line(
                43, 190, 63, 170, fill="#333333", width="3"
            )
            layout_dpad_delete4 = can.create_line(
                151, 190, 130, 170, fill="#333333", width="3"
            )
            layout_ovaltopleft = can.create_oval(
                55, 87, 92, 125, fill="#323232", outline="#232323"
            )
            layout_ovaltopright = can.create_oval(
                100, 87, 140, 125, fill="#323232", outline="#323232"
            )
            layout_ovalbottomleft = can.create_oval(
                55, 144, 92, 179, fill="#323232", outline="#323232"
            )
            layout_ovalbottomright = can.create_oval(
                100, 144, 140, 179, fill="#323232", outline="#323232"
            )
            layout_rectangle_bot = can.create_rectangle(
                69, 142, 120, 179, fill="#323232", outline="#323232"
            )
            layout_rectangle_top = can.create_rectangle(
                77, 87, 119, 115, fill="#323232", outline="#323232"
            )
            layout_rectangle_left = can.create_rectangle(
                55, 110, 80, 159, fill="#323232", outline="#323232"
            )
            layout_rectangle_right = can.create_rectangle(
                93, 109, 140, 160, fill="#323232", outline="#323232"
            )
            layout_ovaldpadmiddle = can.create_oval(79, 115, 115, 150, fill="grey")

            draw_up = can.create_oval(
                96, 94, 101, 112, fill=fillcolornorm, outline=outlinecolornorm
            )
            draw_left = can.create_oval(
                60, 130, 77, 136, fill=fillcolornorm, outline=outlinecolornorm
            )
            draw_right = can.create_oval(
                117, 130, 134, 135, fill=fillcolornorm, outline=outlinecolornorm
            )
            draw_down = can.create_oval(
                94, 155, 100, 174, fill=fillcolornorm, outline=outlinecolornorm
            )
            draw_1 = can.create_oval(
                239, 135, 272, 167, fill=fillcolornorm, outline=outlinecolornorm
            )
            draw_2 = can.create_oval(
                302, 135, 334, 167, fill=fillcolornorm, outline=outlinecolornorm
            )
            draw_bottom_right = can.create_line(
                125, 161, 126, 162, arrow="last", fill=fillcolornorm
            )
            draw_bottom_left = can.create_line(
                74, 162, 73, 163, arrow="last", fill=fillcolornorm
            )
            draw_top_right = can.create_line(
                124, 108, 125, 107, arrow="last", fill=fillcolornorm
            )
            draw_top_left = can.create_line(
                73, 108, 72, 107, arrow="last", fill=fillcolornorm
            )

            can.create_text(61, 71, text="C", font="Serif 8", fill="white")
            can.create_text(69, 71, text="O", font="Serif 8", fill="white")
            can.create_text(77, 71, text="N", font="Serif 8", fill="white")
            can.create_text(85, 71, text="T", font="Serif 8", fill="white")
            can.create_text(93, 71, text="R", font="Serif 8", fill="white")
            can.create_text(101, 71, text="O", font="Serif 8", fill="white")
            can.create_text(108, 71, text="L", font="Serif 8", fill="white")
            can.create_text(118, 71, text="P", font="Serif 8", fill="white")
            can.create_text(126, 71, text="A", font="Serif 8", fill="white")
            can.create_text(133, 71, text="D", font="Serif 8", fill="white")
            can.create_text(242, 116, text="1", font="Sans 5 bold", fill="white")
            can.create_text(255, 116, text="S", font="Sans 5 bold", fill="white")
            can.create_text(261, 116, text="T", font="Sans 5 bold", fill="white")
            can.create_text(267, 116, text="A", font="Sans 5 bold", fill="white")
            can.create_text(273, 116, text="R", font="Sans 5 bold", fill="white")
            can.create_text(279, 116, text="T", font="Sans 5 bold", fill="white")
            can.create_text(306, 116, text="2", font="Sans 5 bold", fill="white")

        elif emu == "Genesis 6-button":
            layout_circle1 = can.create_oval(
                8, 110, 119, 227, fill="#171717", outline="#171717"
            )
            layout_circle2 = can.create_oval(
                260, 100, 371, 227, fill="#171717", outline="#171717"
            )
            layout_rectangle_bottom = can.create_rectangle(
                60, 136, 310, 220, fill="#171717", outline="#171717"
            )
            layout_circle1_bottom = can.create_oval(
                111,
                186,
                267,
                245,
                fill=default_chroma_color,
                outline=default_chroma_color,
            )
            layout_circle2_bottom = can.create_oval(
                94,
                194,
                267,
                254,
                fill=default_chroma_color,
                outline=default_chroma_color,
            )
            layout_circle3_bottom = can.create_oval(
                92,
                195,
                267,
                254,
                fill=default_chroma_color,
                outline=default_chroma_color,
            )

            layout_circle4_bottom = can.create_oval(
                129,
                193,
                294,
                284,
                fill=default_chroma_color,
                outline=default_chroma_color,
            )
            layout_arc1_right = can.create_arc(
                245, 71, 351, 220, fill="#171717", outline="#171717"
            )
            layout_arc2_right = can.create_arc(
                245, 72, 353, 220, fill="#171717", outline="#171717"
            )
            layout_arc3_right = can.create_arc(
                249, 76, 355, 224, fill="#171717", outline="#171717"
            )
            layout_arc4_right = can.create_arc(
                252, 77, 358, 227, fill="#171717", outline="#171717"
            )
            layout_arc5_right = can.create_arc(
                255, 81, 361, 230, fill="#171717", outline="#171717"
            )
            layout_arc6_right = can.create_arc(
                257, 84, 364, 232, fill="#171717", outline="#171717"
            )

            layout_circle_top = can.create_oval(
                42, 40, 338, 185, fill="#171717", outline="#171717"
            )

            layout_circle_left1 = can.create_oval(
                25, 74, 147, 203, fill="#171717", outline="#171717"
            )
            layout_circle_left2 = can.create_oval(
                19, 83, 147, 203, fill="#171717", outline="#171717"
            )
            layout_circle_left3 = can.create_oval(
                14, 95, 130, 203, fill="#171717", outline="#171717"
            )
            layout_hat_bg1 = can.create_oval(
                61, 84, 136, 157, fill="#141414", outline="#141414"
            )
            layout_hat_bg2 = can.create_oval(
                66, 87, 131, 153, fill="#202020", outline="#202020"
            )
            layout_hat_vertical = can.create_rectangle(
                91, 89, 105, 150, fill=fillcolornorm, outline=fillcolornorm
            )
            layout_hat_horizontal = can.create_rectangle(
                69, 113, 128, 126, fill=fillcolornorm, outline=fillcolornorm
            )
            layout_circle_middle = can.create_oval(
                92, 114, 103, 125, fill="#303030", outline="#303030"
            )
            draw_left = can.create_rectangle(
                69, 113, 89, 126, fill=fillcolornorm, outline=outlinecolornorm
            )
            draw_right = can.create_rectangle(
                107, 113, 128, 126, fill=fillcolornorm, outline=outlinecolornorm
            )
            draw_up = can.create_rectangle(
                91, 89, 105, 111, fill=fillcolornorm, outline=outlinecolornorm
            )
            draw_down = can.create_rectangle(
                91, 128, 105, 150, fill=fillcolornorm, outline=outlinecolornorm
            )
            draw_a = can.create_oval(
                236, 135, 265, 164, fill=fillcolornorm, outline=fillcolornorm
            )
            draw_b = can.create_oval(
                269, 117, 298, 146, fill=fillcolornorm, outline=fillcolornorm
            )
            draw_c = can.create_oval(
                305, 107, 334, 136, fill=fillcolornorm, outline=fillcolornorm
            )
            draw_x = can.create_oval(
                229, 101, 250, 122, fill=fillcolornorm, outline=fillcolornorm
            )
            draw_y = can.create_oval(
                258, 87, 279, 108, fill=fillcolornorm, outline=fillcolornorm
            )
            draw_z = can.create_oval(
                289, 79, 310, 100, fill=fillcolornorm, outline=fillcolornorm
            )
            draw_start = [
                can.create_rectangle(
                    174, 107, 196, 115, fill=fillcolornorm, outline=fillcolornorm
                ),
                can.create_oval(
                    172, 107, 179, 115, fill=fillcolornorm, outline=fillcolornorm
                ),
                can.create_oval(
                    192, 107, 200, 115, fill=fillcolornorm, outline=fillcolornorm
                ),
            ]
            can.create_text(251, 149, text="A", font="Sans 13", fill="#303030")
            can.create_text(284, 132, text="B", font="Sans 13", fill="#303030")
            can.create_text(320, 122, text="C", font="Sans 13", fill="#303030")
            can.create_text(240, 112, text="X", font="Sans 11", fill="#303030")
            can.create_text(269, 98, text="Y", font="Sans 11", fill="#303030")
            can.create_text(300, 90, text="Z", font="Sans 11", fill="#303030")
            can.create_text(178, 123, text="S", font="Sans 5", fill="white")
            can.create_text(183, 123, text="T", font="Sans 5", fill="white")
            can.create_text(187, 123, text="A", font="Sans 5", fill="white")
            can.create_text(192, 123, text="R", font="Sans 5", fill="white")
            can.create_text(197, 123, text="T", font="Sans 5", fill="white")

    else:

        for i in BottomInsideFrameRight.winfo_children():
            i.destroy()

        for i in BottomInsideFrameLeft.winfo_children():
            i.destroy()

        can.delete("all")

        can.create_text(
            191,
            130,
            text="No joysticks found.\nPlease connect a device.",
            font="Sans 20 bold",
            fill="deeppink",
        )

    number_of_numhats = numhats.get()

    while show_events.get():

        sleep(0.050)

        for e in event.get(eventtype=allowed_event_types):

            etype = e.type

            event_button = e.dict.get("button")
            event_value = e.dict.get("value")
            event_axis = e.dict.get("axis")

            if (
                etype == constants.JOYDEVICEADDED
                and e.dict.get("device_index") not in range(joy_count)
                or etype == constants.JOYDEVICEREMOVED
            ):
                init()

            if (
                etype == constants.JOYBUTTONDOWN
                or etype == constants.JOYHATMOTION
                or etype == constants.JOYAXISMOTION
            ):
                fillcolor = default_fill_color
                outlinecolor = default_outline_color

            if etype == constants.JOYBUTTONUP:
                fillcolor = fillcolornorm
                outlinecolor = outlinecolornorm

            if number_of_numhats > 0:
                press_dpad()
            else:
                press_axis()

            press_buttons()

        tkgui.update()


TopFrame = Frame(tkgui)
TopFrame.config()
TopFrame.pack(fill="x")

BottomFrame = Frame(tkgui)
BottomFrame.config()
BottomFrame.pack(fill="x")

JoystickFrame = Frame(TopFrame)
JoystickFrame.pack(side="right", anchor="ne")

SelectionFrame = Frame(TopFrame)
SelectionFrame.pack(anchor="c", fill="both")

QuantityLabel = Label(SelectionFrame, textvariable=quantity_joys_label)
QuantityLabel.config(font="sans 10 bold")
QuantityLabel.pack(pady="25")

Butt1 = Radiobutton(SelectionFrame)
Butt1.config(
    variable=selected_joystick,
    value=0,
    command=init,
    textvariable=joy0,
    font="sans 9",
)
Butt1.pack(pady="5", anchor="w")

Butt2 = Radiobutton(SelectionFrame)
Butt2.config(
    variable=selected_joystick,
    value=1,
    command=init,
    textvariable=joy1,
    font="sans 9",
)
Butt2.pack(pady="5", anchor="w")

Butt3 = Radiobutton(SelectionFrame)
Butt3.config(
    variable=selected_joystick,
    value=2,
    command=init,
    textvariable=joy2,
    font="sans 9",
)
Butt3.pack(pady="5", anchor="w")

Butt4 = Radiobutton(SelectionFrame)
Butt4.config(
    variable=selected_joystick,
    value=3,
    command=init,
    textvariable=joy3,
    font="sans 9",
)
Butt4.pack(pady="5", anchor="w")

EmuSelect = OptionMenu(SelectionFrame, emu_name, *emulators, command=init)
EmuSelect.pack(side="left", pady="15")

can = Canvas(JoystickFrame)
can.config(bg=default_chroma_color)
can.pack()

BottomInsideFrameLeft = Label(BottomFrame)
BottomInsideFrameLeft.config()
BottomInsideFrameLeft.pack(side="left", anchor="sw", fill="y")

BottomInsideFrameRight = Label(BottomFrame)
BottomInsideFrameRight.config()
BottomInsideFrameRight.pack(side="right", anchor="ne", fill="y")

tkgui.protocol("WM_DELETE_WINDOW", ask_quit)

if __name__ == "__main__":
    init()
